require 'beemusic_api/version'
require 'rest-client'

module BeemusicApi
  class HttpClient
    def initialize(server_url, debug = true)
      @server_url = server_url
      @debug      = debug
    end

    def authenticate(user, password, session = {})
      unless session.empty?
        @cookies  = session
        @user     = user
        @password = password

        return session
      end

      response =  post(
                    "/auth/credentials",
                    {
                      UserName: user,
                      Password: password,
                      RememberMe: true
                    }
                  )

      if response.valid?
        @cookies  = response.cookies
        @user     = user
        @password = password
      end

      response.cookies
    end

    def put(url, params)
      if @debug
        puts "url = #{url}"
        puts "params = #{params}"
      end

      headers = {
        content_type: :json,
        accept: :json
      }

      headers.merge!(cookies: @cookies) unless @cookies.nil?
      puts "headers = #{headers}"

      RequestResponse.new(
        RestClient.put(
          "#{@server_url}/#{url}",
          params.to_json,
          headers
        ) do |response, request, result|
          response
        end
      )

    end

    def products(per_page: 10, page: 1)
      r = JSON.parse(get("/productpaging/#{per_page}/#{page}"))
      BeemusicApi::Models::Product.build(self, r['Products'], total_entries: r['ItemsCount'].to_i, current_page: r['CurrentPage'].to_i)
    end

    def product(id)
      r = JSON.parse(get("/product/#{id}"))
      BeemusicApi::Models::Product.build(self, r)
    end

    def upload_binary(url, payload)
      r = Faraday.new(@server_url, :ssl => {:verify => false}) do |conn|
        conn.request :multipart
        conn.request :url_encoded
        conn.response :logger
        conn.adapter :net_http
        conn.headers['Cookie'] = @cookies.collect{|k, v| "#{k}=#{v}"}.join('; ')
        conn.basic_auth(@user, @password)
      end

      r.post(url, payload)
    end

    def delete(url, params = {})
      if @debug
        puts "url = #{url}"
        puts "params = #{params}"
      end

      params.merge!({
        params: params[:params].to_json,
        content_type: :json,
        accept:       :json
      })

      params.merge!(cookies: @cookies) unless @cookies.nil?

      RequestResponse.new(RestClient.delete("#{@server_url}/#{url}", params)).response
    end

    def get(url, params = {})
      if @debug
        puts "url = #{url}"
        puts "params = #{params}"
      end

      params.merge!({
        params: params[:params].to_json,
        content_type: :json,
        accept:       :json
      })

      params.merge!(cookies: @cookies) unless @cookies.nil?

      RequestResponse.new(RestClient.get("#{@server_url}/#{url}", params)).response
    end

    def post(url, params)
      if @debug
        puts "url = #{url}"
        puts "params = #{params}"
      end

      headers = {
        content_type: :json,
        accept: :json
      }

      headers.merge!(cookies: @cookies) unless @cookies.nil?
      puts "headers = #{headers}"

      RequestResponse.new(
        RestClient.post(
          "#{@server_url}/#{url}",
          params.to_json,
          headers
        ) do |response, request, result|
          response
        end
      )
    end

    class RequestResponse
      def initialize(response, debug = true)
        @response = response
        @debug    = true
      end

      def cookies
        @response.cookies
      end

      def valid?
        @response.code == 200
      end

      def code
        @response.code
      end

      def message
        response_body['ResponseStatus']['Message'] rescue nil
      end

      def stacktrace
        response_body['ResponseStatus']['StackTrace'] rescue nil
      end

      def errors
        response_body['ResponseStatus']['Errors'] rescue nil
      end

      def response
        @response
      end

      private
      def response_body
        @body ||= JSON.parse(@response.body)
      end
    end
  end
end
